package cn.kk.spring_simple_operation.service.impl;

import cn.kk.spring_simple_operation.entity.APlusProduct;
import cn.kk.spring_simple_operation.entity.SyncProductPublish;
import cn.kk.spring_simple_operation.entity.VisualDesigner;
import cn.kk.spring_simple_operation.mapper.APlusProductMapper;
import cn.kk.spring_simple_operation.mapper.SyncProductPublishMapper;
import cn.kk.spring_simple_operation.mapper.VisualDesignerMapper;
import cn.kk.spring_simple_operation.model.dto.SkuPeopleNameDto;
import cn.kk.spring_simple_operation.service.APlusProductService;
import cn.kk.spring_simple_operation.service.VisualDesignLogService;
import cn.kk.spring_simple_operation.utils.DateUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.List;

/**
 * <p>
 * A+产品 服务实现类
 * </p>
 *
 * @author guokunkun
 * @since 2025-03-18
 */
@Service
@Slf4j
public class APlusProductServiceImpl extends ServiceImpl<APlusProductMapper, APlusProduct> implements APlusProductService {

	@Resource
	private VisualDesignLogService visualDesignLogService;

	@Resource
	private SyncProductPublishMapper syncProductPublishMapper;

	@Resource
	private VisualDesignerMapper visualDesignerMapper;

	@Override
	public void setAPlusProductUploadInfo(List<SkuPeopleNameDto> list) {

		int count = 0;
		Integer currentTimeSecond = DateUtils.getCurrentTimeSecond();
		for (SkuPeopleNameDto skuPeopleNameDto : list) {
			List<APlusProduct> aPlusProducts = baseMapper.selectList(Wrappers.<APlusProduct>lambdaQuery()
					.eq(APlusProduct::getSku, skuPeopleNameDto.getSku())
					.eq(APlusProduct::getSite, "Amazon.com")
					.select(APlusProduct::getId, APlusProduct::getSku, APlusProduct::getSite, APlusProduct::getAPlusPlanId)
			);
			if (CollectionUtils.isEmpty(aPlusProducts)) {
				log.error("sku:{}, site:{}, 未找到对应的A+产品", skuPeopleNameDto.getSku(), "Amazon.com");
				continue;
			}
			APlusProduct aPlusProduct = aPlusProducts.get(0);
			List<APlusProduct> aPlusProductList = baseMapper.selectList(Wrappers.<APlusProduct>lambdaQuery()
					.eq(APlusProduct::getAPlusPlanId, aPlusProduct.getAPlusPlanId())
					.select(APlusProduct::getId, APlusProduct::getSku, APlusProduct::getSite));
			aPlusProductList.add(aPlusProduct);
			for (APlusProduct plusProduct : aPlusProductList) {
				List<SyncProductPublish> syncProductPublishes = syncProductPublishMapper.selectList(Wrappers.<SyncProductPublish>lambdaQuery()
						.eq(SyncProductPublish::getSku, plusProduct.getSku())
						.eq(SyncProductPublish::getSite, plusProduct.getSite())
						.select(SyncProductPublish::getThirtyDaySales)
				);
				if (CollectionUtils.isEmpty(syncProductPublishes)) {
					log.error("sku:{}, site:{}, 未找到对应的销量数据", skuPeopleNameDto.getSku(), "Amazon.com");
					continue;
				}
				SyncProductPublish syncProductPublish = syncProductPublishes.get(0);
				plusProduct.setSale30(syncProductPublish.getThirtyDaySales());
			}
			List<VisualDesigner> visualDesigners = visualDesignerMapper.selectList(Wrappers.<VisualDesigner>lambdaQuery()
					.eq(VisualDesigner::getUserName, skuPeopleNameDto.getPeopleName())
					.in(VisualDesigner::getDepartmentKey, "a_plus_plan")
			);
			if (CollectionUtils.isEmpty(visualDesigners)) {
				log.error("userName:{}, 未找到对应的视觉设计师", skuPeopleNameDto.getPeopleName());
				continue;
			}
			VisualDesigner visualDesigner = visualDesigners.get(0);

			for (APlusProduct plusProduct : aPlusProductList) {
				plusProduct
						.setIsUploadBackstage(2)
						.setUploadBackstageId(Long.valueOf(visualDesigner.getUserId()))
						.setUploadBackstageUser(visualDesigner.getUserName())
						.setUploadBackstageTime(currentTimeSecond)
				;
				baseMapper.updateById(plusProduct);
				visualDesignLogService.addLog("a_plus_product",
						String.format("sku:%s, site:%s, 上传A+产品信息修改为：上传人：%s，上传时间：%s", plusProduct.getSku(), plusProduct.getSite(), visualDesigner.getUserName(), DateUtils.format10YMD(currentTimeSecond)),
						plusProduct.getId());
				count++;
			}

		}
		log.info("共上传{}个A+产品信息", count);

	}
}
